@@ -19,11 +19,20 @@ import time
 import sys
 
 
-__all__ = ["Comm", "DataReceiver", "CommInterface", "SerialInterface", "Printer", "Gateway"]
+__all__ = ["com_interfaces", "Comm", "DataReceiver", "CommInterface", "SerialInterface", "Printer", "Gateway"]
 
 DEFAULT_BAUD = 9600
 
 
+def com_interfaces() -> Dict[str, str]:
+    """
+    Returns a dict containing information about the devices connected through the comport
+    Returns:
+
+    """
+    return {ifc.device: ifc.description for ifc in list_ports.comports()}
+
+
 class Comm(Enum):
     """
     Enum that ties in with the action parameter in the DataReceiver/Source
@@ -123,6 +132,19 @@ class CommInterface(ABC, DataSource):
         """
         pass
 
+    @abstractmethod
+    def send(self, msg: str) -> None:
+        """
+        Forwards a message to the device identified by this
+        interface. May not always be implemented.
+        Args:
+            msg (str): the message to forward to the device.
+
+        Returns:
+            None
+        """
+        pass
+
     @abstractmethod
     def listen(self) -> None:
         """
@@ -155,7 +177,6 @@ class SerialInterface(CommInterface):
             baud_rate (int): the baud rate, the default value is specified in DEFAULT_BAUD = 9600.
         """
         super().__init__()
-        self._ifc_id = "serial_{}".format(port)
         self._port: str = port
         self._baud_rate: int = baud_rate or DEFAULT_BAUD
         self._listening: bool = False
@@ -166,7 +187,7 @@ class SerialInterface(CommInterface):
 
     @property
     def ifc_id(self):
-        return self._ifc_id
+        return self._port
 
     def _poll(self) -> None:
         """
@@ -193,6 +214,9 @@ class SerialInterface(CommInterface):
         self._listening = False
         self.notify(action=Comm.CLOSING)
 
+    def send(self, msg: str) -> None:
+        self._serial.write(msg.encode())
+
     def listen(self) -> None:
         Thread(target=self._poll).start()
 
@@ -335,6 +359,34 @@ class Gateway:
         for interface in self._interfaces:
             self._interfaces[interface].close()
 
+    def forward(self, dest: str, msg: str) -> None:
+        """
+        Forwards a string message to the device connected through the interface identified by dest.
+        Args:
+            dest: the id of the interface to which you're forwarding data.
+            msg: the message being sent.
+
+        Returns:
+            None
+        """
+        try:
+            self._interfaces[dest].send(msg)
+        except KeyError:
+            raise KeyError("No such interface!")
+
+    def broadcast(self, msg: str) -> None:
+        """
+        Broadcasts a string message to each interface connected.
+        Args:
+            msg: the message being broadcasted.
+
+        Returns:
+            None
+        """
+
+        for interface_id in self._interfaces:
+            self._interfaces[interface_id].send(msg)
+
     def listen(self):
         """
         Non blocking listen method for the gateway.
